@@ -25,8 +25,7 @@ package Data::FormValidator;
 
 use vars qw( $VERSION );
 
-$VERSION = 1.9;
-
+$VERSION = '1.91';
 
 require Exporter;
 @ISA = qw(Exporter);
@@ -111,10 +110,14 @@ on input profile.
 
 =head1 SYNOPSIS
 
-In an HTML::Empberl page:
-
     use Data::FormValidator;
 
+	# For the common case of a validating a single profile provided through a hash reference,
+	# using 'validate' like this is the simplest solution 
+	my ($valids, $missings, $invalids, $unknowns) 
+		= Data::FormValidator->validate(\%fdat, \%profile);
+
+    # This is an example of using a validation profile defined in a seperate file
     my $validator = new Data::FormValidator( "/home/user/input_profiles.pl" );
     my ( $valid, $missing, $invalid, $unknown ) = $validator->validate(  \%fdat, "customer_infos" );
 
@@ -158,7 +161,8 @@ sub new {
 	$profile_file	= undef;
     }
 
-    bless { profile_file => $profile_file,
+    bless { 
+		profile_file => $profile_file,
 	    profiles	 => $profiles,
 	  }, $class;
 }
@@ -191,6 +195,12 @@ Here is an example of a valid input profiles specification :
 		[ qw( fullname phone email address) ],
             required_regexp => '/city|state|zipcode/',
             optional_regexp => '/_province$/',
+
+		require_some => {
+			# require any two fields from this group
+			city_or_state_or_zipcode [ 2, qw/city state zipcode/ ], 
+		},
+			
 	    constraints  =>
 		{
 		    email	=> "email",
@@ -227,6 +237,9 @@ Here is an example of a valid input profiles specification :
 	      }
 	    filters       => [ "trim" ],
 	    field_filters => { cc_no => "digit" },
+	    field_filter_regexp_map => {
+			'/_name$/'	=> 'ucfirst',
+	    },
 	},
     }
 
@@ -250,6 +263,24 @@ This is an array reference which contains the name of the fields which
 are required. Any fields in this list which are not present in the
 user input will be reported as missing.
 
+=item required_regexp
+
+This is a regular expression used to specify additional fieds which are
+required. For example, if you wanted all fields names that begin with I<user_> 
+to be required, you could use the regular expression, /^user_/
+
+=item require_some
+
+This is a reference to a hash which defines groups of fields where 
+1 or more field from the group should be required, but exactly
+which fields doesn't matter. The keys in the hash are the group names. 
+These are returned as "missing" unless the required number of fields
+from the group has been filled in. The values in this hash are
+array references. The first element in this hash should be the 
+number of fields in the group that is required. If the first
+first field in the array is not an a digit, a default of "1" 
+will be used. 
+
 =item optional
 
 This is an array reference which contains the name of optional fields.
@@ -257,12 +288,6 @@ These are fields which MAY be present and if they are, they will be
 check for valid input. Any fields not in optional or required list
 will be reported as unknown.
 
-=item required_regexp
-
-This is a regular expression used to specify additional fieds which are
-required. For example, if you wanted all fields names that begin with I<user_> 
-to be required, you could use the regular expression, /^user_/
-
 =item optional_regexp
 
 This is a regular expression used to specify additional fieds which are
@@ -311,6 +336,16 @@ filters which will be applied to specific input fields. The key of the
 hash is the name of the input field and the value is a reference to an
 array of filters, the same way the filters parameter works.
 
+=item field_filter_regexp_map
+
+This is a hash reference where the keys are the regular expressions to
+use and the values are references to arrays of filters which will be
+applied to specific input fields. Used to apply filters to fields that
+match a regular expression. For example, you could make the first letter
+uppercase of all fields that end in "_name" by using the key '/_name$/'
+and the value "ucfirst".
+
+
 =item constraints
 
 This is a reference to an hash which contains the constraints that
@@ -426,14 +461,16 @@ sub validate {
     my ( $self, $data, $name ) = @_;
 
     my $profile;
-    if ( ref $name ) {
-	$profile = $name;
-    } else {
-	$self->load_profiles;
-	$profile = $self->{profiles}{$name};
-	die "No such profile $name\n" unless $profile;
-    }
-    die "Invalid input profile\n" unless ref $profile eq "HASH";
+	if ( ref $name ) {
+		$profile = $name;
+	} else {
+		$self->load_profiles;
+		$profile = $self->{profiles}{$name};
+		die "No such profile $name\n" unless $profile;
+	}
+
+	# check the profile syntax or die with an error. 
+	_check_profile_syntax($profile);
 
     # Copy data and assumes that all is valid
     my %valid	    = %$data;
@@ -466,6 +503,24 @@ sub validate {
 		}
     }   
 
+	# add in specific filters from the regexp map
+	while ( my ($re,$filters) = each %{$profile->{field_filter_regexp_map} }) {
+		my $sub = eval 'sub { $_[0] =~ '. $re . '}';
+		die "Error compiling regular expression $re: $@" if $@;
+
+		foreach my $filter ( _arrayify($filters)) {
+			if (defined $filter) {
+				# Qualify symbolic references
+				$filter = ref $filter ? $filter : "filter_" . $filter;
+				no strict 'refs';
+
+				# find all the keys that match this RE and apply filters to them
+				map { $valid{$_} = $filter->( $valid{$_} ) }
+					grep { $sub->($_) } (keys %valid);
+			}	
+		}
+	}
+ 
     my %required    = map { $_ => 1 } _arrayify($profile->{required});
     my %optional    = map { $_ => 1 } _arrayify($profile->{optional});
 
@@ -485,14 +540,22 @@ sub validate {
 		  $optional{$k} =  1;
        }
     }
+
+	# handle "require_some"
+	my %require_some;
+ 	while ( my ( $field, $deps) = each %{$profile->{require_some}} ) {
+        foreach my $dep (_arrayify($deps)){
+             $require_some{$dep} = 1;
+        }
+    }
+
 	
 	# Remove all empty fields
 	foreach my $field ( keys %valid ) {
 		delete $valid{$field} unless length $valid{$field};
 	}
 
-    # Check if the presence of some fields makes other optional
-    # fields required.
+    # Check if the presence of some fields makes other optional fields required.
     while ( my ( $field, $deps) = each %{$profile->{dependencies}} ) {
         if ($valid{$field}) {
             if (ref($deps) eq 'HASH') {
@@ -526,7 +589,7 @@ sub validate {
 
     # Find unknown
     @unknown =
-      grep { not (exists $optional{$_} or exists $required{$_} ) } keys %valid;
+      grep { not (exists $optional{$_} or exists $required{$_} or exists $require_some{$_} ) } keys %valid;
     # and remove them from the list
     foreach my $field ( @unknown ) {
 	delete $valid{$field};
@@ -539,9 +602,21 @@ sub validate {
 
     # Check for required fields
     foreach my $field ( keys %required ) {
-	push @missings, $field unless exists $valid{$field};
+		push @missings, $field unless exists $valid{$field};
     }
 
+	# Check for the absence of require_some fields
+	while ( my ( $field, $deps) = each %{$profile->{require_some}} ) {
+		my $enough_required_fields = 0;
+		my @deps = _arrayify($deps);
+		# num fields to require is first element in array if looks like a digit, 1 otherwise. 
+		my $num_fields_to_require = ($deps[0] =~ m/^\d+$/) ? $deps[0] : 1;
+		foreach my $dep (@deps){
+			$enough_required_fields++ if exists $valid{$dep};
+		}
+		push @missings, $field unless ($enough_required_fields >= $num_fields_to_require);
+	}
+
     # add in the constraints from the regexp map 
     foreach my $re (keys %{ $profile->{constraint_regexp_map} }) {
        my $sub = eval 'sub { $_[0] =~ '. $re . '}';
@@ -610,7 +685,7 @@ sub _arrayify {
 
    if ( ref $val eq 'ARRAY' ) {
 		# if it's a reference, return an array unless it points an empty array. -mls
-                return $val->[0] ? @$val : ();
+                return (length $val->[0]) ? @$val : ();
    } 
    else {
 		# if it's a string, return an array unless the string is missing or empty. -mls
@@ -1055,7 +1130,7 @@ sub valid_phone {
 =item american_phone
 
 This constraints checks if the number is a possible North American style
-of phone number : (XXX) XXX-XXXX. It has to contains more than 7 digits.
+of phone number : (XXX) XXX-XXXX. It has to contains 7 or more digits.
 
 =cut
 
@@ -1181,6 +1256,36 @@ sub valid_ip_address {
    }
 }
 
+# check the profile syntax and die if we have an error
+sub _check_profile_syntax {
+	my $profile = shift;
+
+	die "Invalid input profile: needs to be a hash reference\n" unless ref $profile eq "HASH";
+
+	my @valid_profile_keys = (qw/
+		optional
+		required
+		required_regexp 
+		require_some
+		optional_regexp
+		constraints
+		constraint_regexp_map
+		dependencies
+		dependency_groups
+		defaults
+		filters
+		field_filters
+		field_filter_regexp_map
+		missing_optional_valid /);
+
+	# If any of the keys in the profile are not listed as valid keys here, we die with an error	
+	for my $key (keys %$profile) {
+		unless (grep {$key eq $_} @valid_profile_keys) {
+			die "Invalid input profile: $key is not a valid profile key\n"
+		}
+	}
+}
+
 1;
 
 __END__
@@ -1203,9 +1308,6 @@ The credit card checksum validation was taken from contribution by
 Bruce Albrecht <bruce.albrecht@seag.fingerhut.com> to the MiniVend
 program.
 
-Mark Stosberg contributed a number of enhancements including
-I<required_regexp>, I<optional_regexp> and I<constraint_regexp_map>
-
 =head1 PUBLIC CVS SERVER
 
 Data::FormValidator now has a publicly accessible CVS server provided by
